@@ -224,8 +224,15 @@ def main():
                   default=[],
                   metavar='PATH_OR_URI',
       help = 'The path to an OWL RDF/XML graph to use DLP to extract rules from '+
-      '(other wise, fact graph(s) are used)  ') 
-      
+      '(other wise, fact graph(s) are used)  ')
+
+    op.add_option('--ruleFormat',
+        default='n3',
+        dest='ruleFormat',
+        metavar='RULE_FORMAT',
+        choices = ['n3', 'rif'],
+        help = "The format of the rules to parse ('n3', 'rif').  The default is %default")
+
     op.add_option('--ontologyFormat', 
                 default='xml',
                 dest='ontologyFormat',
@@ -270,9 +277,19 @@ def main():
             userFuncs = imp.load_source('builtins', options.builtins)
             rs = HornFromN3(fileN,
                             additionalBuiltins=userFuncs.ADDITIONAL_FILTERS)
+            nsBinds.update(rs.nsMapping)
+        elif options.ruleFormat == 'rif':
+            try:
+                from FuXi.Horn.RIFCore import RIFCoreParser
+                rif_parser = RIFCoreParser(location=fileN,debug=options.debug)
+                rs = rif_parser.getRuleset()
+            except ImportError, e:
+                raise Exception(
+                    "Missing 3rd party libraries for RIF processing: %s"%e
+                )
         else:
             rs = HornFromN3(fileN)
-        nsBinds.update(rs.nsMapping)
+            nsBinds.update(rs.nsMapping)
         ruleSet.formulae.extend(rs)
         #ruleGraph.parse(fileN,format='n3')
 
@@ -382,11 +399,10 @@ def main():
                         children = [desc for desc in c.subSumpteeIds()]
                         for child in children:
                             for otherChild in [o for o in children if o is not child]:
-                                if not otherChild in [c.identifier 
+                                if not otherChild in [c.identifier
                                           for c in Class(child).disjointWith]:# and\
-                                    warnings.warn("Primitive children (of %s) "+
+                                    warnings.warn("Primitive children (of %s) "%(c.qname) +
                                           "must be mutually disjoint: %s and %s"%(
-                                      c.qname,
                                       Class(child).qname,
                                       Class(otherChild).qname),UserWarning,1)
 #                if not isinstance(c.identifier,BNode):
@@ -486,7 +502,6 @@ def main():
             start = time.time()
             network.feedFactsToAdd(generateTokenSet(magicSeeds))
             if not [rule for rule in factGraph.adornedProgram if len(rule.sip)]:
-                import warnings
                 warnings.warn("Using GMS sideways information strategy with no "+
                               "information to pass from query.  Falling back to "+
                               "naive method over given facts and rules")
