@@ -63,6 +63,17 @@ nameMap = {
   'harsh'          : DDL_STRICTNESS_HARSH,
 }
 
+def ReplaceHybridPredcates(rules,hybridPreds2Replace):
+    for rule in rules:
+        for headLiteral in iterCondition(rule.formula.head):
+            headOp = GetOp(headLiteral)
+            if headOp in hybridPreds2Replace:
+                headLiteral.setOperator(URIRef(headOp + u'_derived'))
+        for bodyAtom in iterCondition(rule.formula.body):
+            bodyPred = GetOp(bodyAtom)
+            if bodyPred in hybridPreds2Replace:
+                bodyAtom.setOperator(URIRef(bodyPred + u'_derived'))
+
 def SetupDDLAndAdornProgram(factGraph,
                             rules,
                             GOALS,
@@ -73,7 +84,6 @@ def SetupDDLAndAdornProgram(factGraph,
                             hybridPreds2Replace=None):
     if not defaultPredicates:
         defaultPredicates = [],[]
-    _dPredsProvided = bool(derivedPreds)
     if not derivedPreds:
         _derivedPreds=DerivedPredicateIterator(factGraph,
                                                rules,
@@ -91,43 +101,56 @@ def SetupDDLAndAdornProgram(factGraph,
                         derivedPreds,
                         ignoreUnboundDPreds,
                         hybridPreds2Replace = hybridPreds2Replace)
-    for hybridPred in hybridPreds2Replace:
-        #If there are hybrid predicates, add rules that derived their IDB counterpart
-        #using information from the adorned queries to determine appropriate arity
-        #and adornment
-        hybridPredFull = URIRef(hybridPred + u'_derived')
-        for hPred,adornment in set([(GetOp(pred),
-                                     ''.join(pred.adornment))
-            for pred in factGraph.queryAtoms.get(hybridPredFull,[])]): 
-            if len(adornment) == 1:
-                # p_derived^{a}(X) :- p(X)
-                body = BuildUnitermFromTuple(
-                                    (Variable('X'),
-                                     RDF.type,
-                                     hybridPred))
-                head = BuildUnitermFromTuple(
-                                    (Variable('X'),
-                                     RDF.type,
-                                     hPred))
-            else:
-                # p_derived^{a}(X,Y) :- p(X,Y)
-                body = BuildUnitermFromTuple(
-                                    (Variable('X'),
-                                     hybridPred,
-                                     Variable('Y')))
-                head = BuildUnitermFromTuple(
-                                    (Variable('X'),
-                                     hPred,
-                                     Variable('Y')))
-            _head=AdornedUniTerm(head,list(adornment))
-            rule=AdornedRule(Clause(And([body]),_head.clone()))
-            rule.sip = Graph()
-            adornedProgram.add(rule)
 
     if factGraph is not None:
         factGraph.adornedProgram = adornedProgram    
     return adornedProgram
 
+def CreateHybridPredicateRule(hybridPred,program,nsMap=None):
+    hPred           = URIRef(hybridPred + u'_derived')
+    literals = set(reduce(lambda l,r: l+r,
+        [list(iterCondition(clause.formula.body))+
+         list(iterCondition(clause.formula.head))
+         for clause in program]))
+    for literal in literals:
+        if GetOp(literal) == hybridPred:
+            noArgs = len(GetArgs(literal))
+            if noArgs == 1:
+                # p_derived(X) :- p(X)
+                body = BuildUnitermFromTuple(
+                    (Variable('X'),
+                     RDF.type,
+                     hybridPred),
+                    newNss=nsMap
+                )
+                head = BuildUnitermFromTuple(
+                    (Variable('X'),
+                     RDF.type,
+                     hPred),
+                    newNss=nsMap
+                )
+                vars = [Variable('X')]
+            else:
+                # p_derived(X,Y) :- p(X,Y)
+                body = BuildUnitermFromTuple(
+                    (Variable('X'),
+                     hybridPred,
+                     Variable('Y')),
+                    newNss=nsMap
+                )
+                head = BuildUnitermFromTuple(
+                    (Variable('X'),
+                     hPred,
+                     Variable('Y')),
+                    newNss=nsMap
+                )
+                vars = [Variable('Y'),Variable('X')]
+            return Rule(
+                Clause(And([body]),head),
+                nsMapping=nsMap,
+                declare=vars
+            )
+
 def MagicSetTransformation(factGraph,
                            rules,
                            GOALS,
@@ -353,6 +376,14 @@ def AdornRule(derivedPreds,
 def BasePredicateFromHybrid(pred):
     return URIRef(pred[:-8])
 
+def IsHybridPredicateRule(rule,derivedSuffix='derived'):
+    fullSuffix = '_'+derivedSuffix
+    basePred = URIRef(GetOp(rule.formula.head).split(fullSuffix)[0])
+    return GetOp(
+        rule.formula.head).find(fullSuffix)+1 and len(
+            list(iterCondition(rule.formula.body)))==1 and \
+            GetOp(list(iterCondition(rule.formula.body))[0]) == basePred
+
 def IsHybridPredicate(pred,hybridPreds2Replace):
     op = GetOp(pred)
     return op[-7:] == 'derived' and op[:-8] in hybridPreds2Replace
@@ -366,9 +397,7 @@ def compareAdornedPredToRuleHead(aPred,head, hybridPreds2Replace):
     assert isinstance(head,Uniterm)
     if head.getArity() == aPred.getArity():
         return headPredicateTerm == aPredTerm or isinstance(headPredicateTerm,
-                                                            Variable) or (
-               IsHybridPredicate(aPred,hybridPreds2Replace) and 
-               aPredTerm[:-8] == headPredicateTerm)
+                                                            Variable)
     return False
 
 def AdornProgram(factGraph,
@@ -393,6 +422,7 @@ def AdornProgram(factGraph,
     from FuXi.DLP import LloydToporTransformation
     from collections import deque
     goalDict = {}
+    handled = set()
     hybridPreds2Replace = hybridPreds2Replace or []
     adornedPredicateCollection=set()
     for goal,nsBindings in NormalizeGoals(goals):
@@ -402,7 +432,7 @@ def AdornProgram(factGraph,
     def unprocessedPreds(aPredCol):
         rt=[]
         for p in aPredCol:
-            if not  p.marked:
+            if p.herbrand_hash not in handled:
                 rt.append(p)
             if p not in goalDict:
                 goalDict.setdefault(GetOp(p),set()).add(p)
@@ -434,11 +464,13 @@ def AdornProgram(factGraph,
                                         AdornLiteral(pred.toRDFTuple(),
                                                      nsBindings,
                                                      pred.naf) or pred
-                        if (GetOp(pred) in derivedPreds or
-                            (GetOp(pred) in hybridPreds2Replace if hybridPreds2Replace
-                            else False)) and aPred not in adornedPredicateCollection:
+                        op = GetOp(pred)
+                        if (op in derivedPreds or
+                            (op in hybridPreds2Replace if hybridPreds2Replace
+                            else False)
+                            ) and aPred not in adornedPredicateCollection:
                             adornedPredicateCollection.add(aPred)
-        term.marked=True
+        handled.add(term.herbrand_hash)#term.marked=True
         toDo.extendleft(unprocessedPreds(adornedPredicateCollection))
         
     factGraph.queryAtoms = goalDict
@@ -448,13 +480,16 @@ class AdornedUniTerm(Uniterm):
     def __init__(self,uterm,adornment=None,naf = False):
         self.marked = False
         self.adornment=adornment
-        self.nsMgr=uterm.nsMgr
-        newArgs=copy.deepcopy(uterm.arg)
-        super(AdornedUniTerm, self).__init__(uterm.op,newArgs,naf=naf)
+        self.nsMgr=GetUterm(uterm).nsMgr
+        newArgs=copy.deepcopy(GetUterm(uterm).arg)
+        super(AdornedUniTerm, self).__init__(GetUterm(uterm).op,newArgs,naf=naf)
         self.isMagic=False
 
     def clone(self):
         return AdornedUniTerm(self,self.adornment,self.naf)
+        
+    def convert2NormalUterm(self):
+        return buildUniTerm(self.toRDFTuple())
                 
     def makeMagicPred(self):
         """
@@ -495,16 +530,17 @@ class AdornedUniTerm(Uniterm):
                     return True
         return False
                 
-    def getDistinguishedVariables(self,varsOnly=False):
+    def getDistinguishedVariables(self,varsOnly=False,bound=True):
+        adornment2Compare = 'b' if bound else 'f'
         if self.op == RDF.type:
             for idx,term in enumerate(GetArgs(self)):
-                if self.adornment[idx]=='b':
+                if self.adornment[idx]==adornment2Compare:
                     if not varsOnly or isinstance(term,Variable):
                         yield term
         else:
             for idx,term in enumerate(self.arg):
                 try:
-                    if self.adornment[idx]=='b':
+                    if self.adornment[idx]==adornment2Compare:
                         if not varsOnly or isinstance(term,Variable):
                             yield term
                 except IndexError: pass
