@@ -21,8 +21,8 @@ from AlphaNode import AlphaNode, ReteToken, SUBJECT, PREDICATE, OBJECT, BuiltInA
 from BuiltinPredicates import FILTERS
 from FuXi.Horn import ComplementExpansion, DATALOG_SAFETY_NONE, \
                       DATALOG_SAFETY_STRICT, DATALOG_SAFETY_LOOSE
-from FuXi.Syntax.InfixOWL import *
-from FuXi.Horn.PositiveConditions import Uniterm, SetOperator, Exists, Or
+from FuXi.Syntax.InfixOWL import Class, Property, Individual
+from FuXi.Horn.PositiveConditions import Uniterm, SetOperator, Exists, Or, GetUterm
 from FuXi.DLP import MapDLPtoNetwork,non_DHL_OWL_Semantics,IsaFactFormingConclusion
 from FuXi.DLP.ConditionalAxioms import AdditionalRules
 from Util import generateTokenSet,renderNetwork
@@ -146,7 +146,7 @@ class InferredGoal(Exception):
     def __init__(self, msg):
         self.msg = msg
     def __repr__(self):
-        return "Goal inferred!: %"%self.msg         
+        return "Goal inferred!: %s"%self.msg
 
 class ReteNetwork:
     """
@@ -160,7 +160,9 @@ class ReteNetwork:
                  nsMap = {},
                  graphVizOutFile=None,
                  dontFinalize=False,
-                 goal=None):
+                 goal=None,
+                 rulePrioritizer=None,
+                 alphaNodePrioritizer=None):
         self.leanCheck = {}
         self.goal = goal        
         self.nsMap = nsMap
@@ -189,6 +191,8 @@ class ReteNetwork:
         if not dontFinalize:
             self.ruleStore._finalize()
         self.filteredFacts = Graph()
+        self.rulePrioritizer      = rulePrioritizer
+        self.alphaNodePrioritizer = alphaNodePrioritizer
         
         #'Universal truths' for a rule set are rules where the LHS is empty.  
         # Rather than automatically adding them to the working set, alpha nodes are 'notified'
@@ -515,13 +519,16 @@ class ReteNetwork:
                     if not isinstance(rule.formula.body,Exists) or \
                        bN not in rule.formula.body.declare:
                        BNodeReplacement[bN] = BNode()
-        for rhsTriple in termNode.consequent:
+        consequents = self.rulePrioritizer(termNode.consequent
+            ) if self.rulePrioritizer else termNode.consequent
+        for rhsTriple in consequents:
             if BNodeReplacement:
                 rhsTriple = tuple([BNodeReplacement.get(term,term) for term in rhsTriple])
             if debug:
                 if not tokens.bindings:
-                    tokens._generateBindings()                    
-            override,executeFn = termNode.executeActions.get(rhsTriple,(None,None))
+                    tokens._generateBindings()
+            key = tuple(map(lambda item: None if isinstance(item,BNode) else item,rhsTriple))
+            override,executeFn = termNode.executeActions.get(key,(None,None))
             
             if override:
                 #There is an execute action associated with this production
@@ -548,18 +555,28 @@ class ReteNetwork:
                         if debug:
                             print "Inferred triple: ", inferredTriple, " from ",termNode.clauseRepresentation()
                             inferredToken.debug = True
-                        self.inferredFacts.add(inferredTriple)
-                        self.addWME(inferredToken)
-                        currIdx = self.instanciations.get(termNode,0)
-                        currIdx+=1
-                        self.instanciations[termNode] = currIdx
                         if executeFn:
                             #The indicated execute action is supposed to be triggered
                             #when the indicates RHS triple is inferred for the
                             #first time
                             executeFn(termNode,inferredTriple,tokens,binding,debug)
-                        if self.goal is not None and self.goal in self.inferredFacts:
-                           raise InferredGoal("Proved goal " + repr(self.goal))                    
+                        if self.goal is not None and self.goal == inferredTriple:#in self.inferredFacts:
+                            for binding in tokens.bindings:
+                                rt=None#self.bfp.extractProof(binding,termNode.ruleNo,inferredTriple,applyBindings=True)
+                                def depth(l):
+                                    assert isinstance(l,(tuple))
+                                    if isinstance(l,tuple):
+                                        depths = [depth(item) for item in l[2]]
+                                        return 1 + max(depths) if depths else 1
+                                #self.bfp.proof = -1,rt#depth(rt),rt
+                            if debug:
+                                "Proved goal " + repr(self.goal)
+                            raise InferredGoal("Proved goal " + repr(self.goal))                    
+                        self.inferredFacts.add(inferredTriple)
+                        self.addWME(inferredToken)
+                        currIdx = self.instanciations.get(termNode,0)
+                        currIdx+=1
+                        self.instanciations[termNode] = currIdx                            
                     else:
                         if debug:
                             print "Inferred triple skipped: ", inferredTriple
@@ -584,14 +601,22 @@ class ReteNetwork:
             if alpha-mem then alpha-memory-activation (alpha-mem, w)
         end        
         """
-#        print wme.asTuple()       
-        for termComb,termDict in iteritems(self.alphaPatternHash):
-            for alphaNode in termDict.get(wme.alphaNetworkHash(termComb),[]):
-#                print "\t## Activated AlphaNode ##"
-#                print "\t\t",termComb,wme.alphaNetworkHash(termComb)
-#                print "\t\t",alphaNode
-                alphaNode.activate(wme.unboundCopy())
-    
+#        print wme.asTuple()
+        #If the user provided a function that enforces an ordering in the
+        # evaluation of alpha nodes, then we use this ordering or the 'natural'
+        # ordering otherwise
+        aNodes = reduce(
+            lambda l,r:l+r,
+            filter(lambda i:i,
+                   map(lambda (termComb,termDict): termDict.get(wme.alphaNetworkHash(termComb),[]) ,
+                       iteritems(self.alphaPatternHash))),[])
+        sortedANodes = self.alphaNodePrioritizer(aNodes) if self.alphaNodePrioritizer else aNodes
+        for alphaNode in sortedANodes:
+#            print "\t## Activated AlphaNode ##"
+#            print "\t\t",termComb,wme.alphaNetworkHash(termComb)
+#            print "\t\t",alphaNode
+            alphaNode.activate(wme.unboundCopy())
+
     def feedFactsToAdd(self,tokenIterator):
         """
         Feeds the network an iterator of facts / tokens which are fed to the alpha nodes 
@@ -653,9 +678,12 @@ class ReteNetwork:
         """
         for tNode in self.terminalNodes:
             for rule in tNode.rules:
-                if not isinstance(rule.formula.head, Uniterm):
+                if not isinstance(rule.formula.head, (Exists,Uniterm)):
                     continue
-                headTriple = rule.formula.head.toRDFTuple()
+                headTriple = GetUterm(rule.formula.head).toRDFTuple()
+                headTriple = tuple(
+                    map(lambda item: None if isinstance(item,BNode) else item,
+                        headTriple))
                 tNode.executeActions[headTriple] = (override,executeFn)
     
     def buildNetwork(self,lhsIterator,rhsIterator,rule,aFilter=False):
